<?php
/*
** @name: Controller_plan.php
**
** @description: PHP controller for common plan componnents to use in any plan types
** @author: Quentin George <q.george@esia-sa.com>
** @date: 19/09/2018
*/
$path = $_SERVER['DOCUMENT_ROOT'];
// include_once $path . '/common.php';
include_once $path . '/Model_adm.php';
include_once $path . '/plugin/plan/Model_plan.php';

class Controller_Plan
{
    const FOLDERS_CONTROLLER = 'controllers/folder_control.php';
    const DATA_PLANS = '/data/plan/plans/';
    const DATA_ICONS = '/data/plan/icons/';
    const ALERTS_STATES = array('OK' => 0, 'warning' => 1, 'critical' => 2, 'unknown' => 3);
    const ALERTS_PRIORITIES = array(
    		self::ALERTS_STATES['critical'],
    		self::ALERTS_STATES['warning'],
    		self::ALERTS_STATES['unknown'],
    		self::ALERTS_STATES['OK']
    );

    // Global class variables setted by class constructor
    protected $dbconnection;
    protected $user;
    public $pluginInfos;
    public $pluginURL;
    public $templatePath;
    public $isUserPerms;
    public $pathDataPlan;
    public $pathDataIcons;

    // Set global class variables
    public function __construct($path, $dbconnection, $user)
    {
        $pluginInfos = Model_esia::getWebPlugin($dbconnection, false, 'WEBP_PLAN')[0];

        $this->dbconnection = $dbconnection;

        $this->pluginInfos = $pluginInfos;
        $this->pluginURL = $pluginInfos['apache_path'];
        $this->templatePath = $path . $pluginInfos['apache_path'] . 'templates/';

        $this->pathDataPlans = $path . self::DATA_PLANS;
        $this->pathDataIcons = $path . self::DATA_ICONS;

        $this->user = $user;
        $this->isUserPerms = array(
            'view' => ($user->getDroits($dbconnection, "map_view") == 1),
            'admin' => ($user->getDroits($dbconnection, "map_administration") == 1),
            'create_folder' => ($user->getDroits($dbconnection, "map_create_folder") == 1),
            'create_plan' => ($user->getDroits($dbconnection, "map_create_plan") == 1),
        );
    }
    /**
    *** @name: checkUserPlanPrivilege
    *** @desc: Check if the current user can modify or only read a plan
	*** @param: (INTEGER) $planId => id of the plan
	*** @retval: (ARRAY of BOOL) => WRITE true if user can modify the plan (privilege WRITE or CREATOR), READ true if user can read the plan
	**/
    public function checkUserPlanPrivilege($planId)
    {
        $userAccess = array(
            'WRITE' => false,
            'READ' => false
        );

        $access = Model_Plan::getPlanPrivilege($this->dbconnection, $planId, $this->user->getId());
        if ($access == 'CREATOR' || $access == 'WRITE')
        {
            $userAccess['WRITE'] = true;
            $userAccess['READ'] = true;
        }
        elseif ($access == 'READ')
        {
            $userAccess['READ'] = true;
        }

        return $userAccess;
    }
    /**
    *** @name: checkUserFolderPrivilege
    *** @desc: Check if the current user can modify or only read a folder
	*** @param: (INTEGER) $folderId => id of the folder
	*** @retval: (ARRAY of BOOL) => WRITE true if user can modify the folder (privilege WRITE or CREATOR), READ true if user can read the folder content
	**/
    public function checkUserFolderPrivilege($folderId)
    {
        $userAccess = array(
            'WRITE' => false,
            'READ' => false
        );

        $access = Model_Plan::getFolderPrivilege($this->dbconnection, $folderId, $this->user->getId());
        if ($access == 'CREATOR' || $access == 'WRITE')
        {
            $userAccess['WRITE'] = true;
            $userAccess['READ'] = true;
        }
        elseif ($access == 'READ')
        {
            $userAccess['READ'] = true;
        }

        return $userAccess;
    }
    /**
    *** @name: setNavBar
    *** @desc: Create breadcrumbs navbar from plan root folder to current plan or folder
    *** @param: (OBJECT) $navbar => navbar class from common.php
    *** @param: (STRING) $rootName => name of the plan plugin ($lang['WEBP_PLAN']) for the plan root link text
	*** @param: (ARRAY) $folders => list of folders parents (Model_plan::getFolders)
    *** @param: (INTEGER) $firstFolderId => if no $plan, id of the current folder
    *** @param: (ARRAY) $plan => current plan (Model_plan::getPlan)
	**/
    public function setNavBar($navbar, $rootName, $folders, $firstFolderId=null, $plan=null)
    {
        $navbarElements = array();
        $elemId = $firstFolderId;

        if (!empty($plan))
        {
            array_push($navbarElements, array('name' => $plan['name'], 'href' => $this->pluginURL . $plan['type_controller'] . '&id=' . $plan['id']));
            $elemId = $plan['folders'];
        }

        while(!empty($elemId))
        {
            if ($folders[$elemId])
            {
                array_unshift($navbarElements, array('name' => $folders[$elemId]['name'], 'href' => $this->pluginURL . self::FOLDERS_CONTROLLER . '?id=' . $elemId));
                $elemId = $folders[$elemId]['parent'];
            }
            else
            {
                $elemId = null;
            }
        }
        // Add Root folder
        array_unshift($navbarElements, array('name' => $rootName, 'href' => $this->pluginURL . self::FOLDERS_CONTROLLER));

        // Display navbar
        foreach($navbarElements as $navElt)
        {
            $navbar->addElement($navElt['name'], $navElt['href'], null);
        }
    }
    /**
    *** @name: getObjectsStateIcon
    *** @desc: Fetch plan objects icons & status
    *** @param: (ARRAY) $path => Server path
    *** @param: (INTEGER) $planId => Id of the plan where to fetch the objects
    *** @retval: (ARRAY) => Objects where each plan object contain his icons & states
	**/
    public function getObjectsStateIcon($path, $planId)
    {
        $objects = Model_plan::getObjectsByPlan($this->dbconnection, $planId);
        foreach($objects as $key => $object)
    	{
    		$objects[$key]['params'] = json_decode($object['params'], true);
    		if($objects[$key]['params']['getstate']=='no')
    		{
    		    $objects[$key]['state'] = "ok";
    		}
    		else
    		{
        		$globalState = Model_Plan::getObjectState($this->dbconnection, $objects[$key], $this->user->getId());
        		$objects[$key]['state'] = $globalState;
    		}
        	$objects[$key]['icon'] = Model_plan::getIcon($this->dbconnection, $object['icon_id'])[0];
        	if(file_exists($path . $objects[$key]['icon']['path'] . $objects[$key]['icon']['name'] . "_warning." . $objects[$key]['icon']['type']))
        	{
        		$objects[$key]['file'] = $objects[$key]['icon']['path'] . $objects[$key]['icon']['name'] . "_" . $objects[$key]['state'] . "." . $objects[$key]['icon']['type'];
        	}
        	else
        	{
        		$objects[$key]['file'] = $objects[$key]['icon']['path'] . $objects[$key]['icon']['name'] . "_ok." . $objects[$key]['icon']['type'];
        		$objects[$key]['uniq'] = true;
        	}
    	}

        return $objects;
    }
    /**
    *** @name: priority_state
    *** @desc: Compare 2 states with the priorities array to get the most critical
    *** @param: (ARRAY) $priorities => List of the priorities to check. Sorted by most critical to less critical
    *** @param: (INTEGER) $a => First state to check
	*** @param: (INTEGER) $b => Second state to check
    *** @retval: (INTEGER) => Most critical state number
	**/
    public function priority_state($priorities, $a, $b)
    {
    	if( $a == $b ) { return $a; } //pas besoin d'aller dans la boucles si ils sont identiques

    	foreach ($priorities as $v)
    	{
    		if ( $v == $a || $v == $b ) { return $v; }
    	}
    }
    /**
    *** @name: getMostCritical
    *** @desc: Get node global state from his services state
    *** @param: (ARRAY) $listState => Number of services in each state for a node
    *** @retval: (STRING) => Most critical state string
	**/
    public function getMostCritical($listState)
    {
    	if($listState['critical'] > 0 ) return 'critical';
    	elseif($listState['warning'] > 0) return 'warning';
    	elseif($listState['unknown'] > 0) return 'unknown';
    	elseif($listState['OK'] > 0) return 'ok';
    	elseif($listState['pending'] > 0) return 'pending';
    	else return 'ok';

    	return 'unknown'; //AUCUN SERVICE
    }
    /**
    *** @name: getAlertsByElement
    *** @desc: Get nodes alerts for elements
    *** @param: (ARRAY) $lang => lang variable
    *** @param: (INTEGER) $planId => id of the plan to check objects alerts
	*** @param: (ARRAY) $alerts => previous alerts founded if use this function recursively
    *** @param: (INTEGER) $objectid => id of plan object
    *** @retval: (ARRAY) => Alerts founded
	**/
    public function getAlertsByElement($lang, $planId, $alerts, $objectid=null)
    {
        if(!$planId && $objectid)
    	{
    		$objects[] = Model_plan::getObject($this->dbconnection, $objectid);
    	}
    	else
    	{
    		$objects = Model_plan::getObjectsByPlan($this->dbconnection, $planId);
    	}
    	foreach($objects as $key => $object)
    	{
    		if(isset($object['type']) && ($object['type'] == 2 || $object['type'] == 3))
    		{
    		    $objects[$key]['params'] = json_decode($object['params'],true);
    		    if($objects[$key]['params']['getstate']=='no')
    		    {
    		        $objects[$key]['state'] = "ok";
    		    }
    		    else
    		    {
        			/*si le type de l'objet est un plan, on va récupérer l'état des objets de ce sous-plan */
        			if($object['type'] == 2)
        			{
        				$alerts = $this->getAlertsByElement($lang, $object['link'], $alerts);
        			}
        			else
        			{
        				
        				$nodes = array_map('intval',explode(",",$objects[$key]['params']['nodes']));
        				if(isset($nodes[0]) && $nodes[0] != 0)
        				{
        					foreach($nodes as $node)
        					{
        						$listState = Model_esia::countNodeState($this->dbconnection, $node);
        						$nodeState = $this->getMostCritical($listState);
    
        						$devType = Model_esia::getDeviceType($this->dbconnection);
        						//on récupère l'état du noeud
        						$errors = Model_esia::getNodeError($this->dbconnection, $node);
        						$state='OK';
        						$list_error =array();
        						foreach($errors as $error)
        						{
        							$common_name = Model_esia::getCommonName($lang, $error['service'], $error['plugin_name']);
        							$list_error[]=array('type'=>$error['type'],
        									'name'=>$error['error'],
        									'service'=>$common_name,
        									'message'=>$error['message'],
        									'time'=>$error['time'],
        							);
        							if($error['type']=='critical')
        							{
        								$state='critical';
        							}
        							elseif($error['type']=='warning' && $state!='critical')
        							{
        								$state='warning';
        							}
        							elseif($error['type']=='unknown' && $state!='warning' && $state!='critical')
        							{
        								$state='unknown';
        							}
        						}
    
        						if(!empty($list_error) && $this->user->getAccess($this->dbconnection, "device_view", $errors[0]['device']) == 1)
        						{
        							$page_control='/node_view.php?name=';
        							if(!empty($devType[$errors[0]['device_type']]['web_plugin_control_page']))
        							{
        								$page_control = Model_esia::replacePageControlVariable($devType[$errors[0]['device_type']]['web_plugin_control_page'], $node);
        								$page_control = $devType[$errors[0]['device_type']]['web_plugin_apache_path'].$page_control;
        							}
    
        							$listNode=array('id'=>$node,
        									'name'=>$errors[0]['device'],
        									'type'=>$errors[0]['device_type'],
        									'page_control'=>$page_control,
        									//'nbrErr'=>count($list_error),
        									'state'=>$state,
        									'errors'=>$list_error);
        							$alerts['nodes'][] = $listNode;
        						}
        						else
        						{
        							$dev = Model_esia::InstanceNode($this->dbconnection, $node);
        							$page_control='/node_view.php?name=';
        							if(!empty($devType[$dev->getType()]['web_plugin_control_page']))
        							{
        								$page_control = Model_esia::replacePageControlVariable($devType[$dev->getType()]['web_plugin_control_page'], $node);
        								$page_control = $devType[$dev->getType()]['web_plugin_apache_path'].$page_control;
        							}
        							if (empty($list_error) && $this->user->getAccess($this->dbconnection,"device_view",$dev->getName()) ==1)
        							{
        								$listNode=array('id'=>$node,
        										'name'=>$dev->getName(),
        										'type'=>$dev->getType(),
        										'page_control'=>$page_control,
        										'state'=>$state,
        										'errors'=>$list_error);
        								$alerts['nodes'][] = $listNode;
        							}
        						}
        					}
        				}
    
        				if(isset($objects[$key]['params']['groups']) && !empty($objects[$key]['params']['groups']))
        				{
        					$groups = array_map('intval',explode(",",$objects[$key]['params']['groups']));
    
        					foreach($groups as $group)
        					{
        						//On récupère l'état des noeuds dans le parent et l'état des sous-groupes.
        						$state_parent =  model_esia::getGroupNodeStates($this->dbconnection, $this->user->getId(), $group,false);
        						$state_child =  model_esia::getGroupNodeStates($this->dbconnection, $this->user->getId(), $group,true);
    
        						reset($state_parent);
        						$listState['ok'] = $state_parent[key($state_parent)]['ok'];
        						$listState['warning'] = $state_parent[key($state_parent)]['warning'];
        						$listState['critical'] = $state_parent[key($state_parent)]['critical'];
        						$listState['unknown'] = $state_parent[key($state_parent)]['unknown'];
        						//On additionne les 2 tableau pour avoir l'état global d'un groupe
        						foreach($state_child as $current)
        						{
        							$listState['ok'] += $current['ok'];
        							$listState['warning'] += $current['warning'];
        							$listState['critical'] += $current['critical'];
        							$listState['unknown'] += $current['unknown'];
        						}
    
        						$groupState = $this->getMostCritical($listState);
    
        						$alerts['groups'][$group]['state'] = $groupState;
        						$alerts['groups'][$group]['id'] = $group;
        						$groupName = Model_esia::getGroupDeviceIn($this->dbconnection,$group)[0]['name'];
        						$alerts['groups'][$group]['name'] = $groupName;
        					}
        				}
        			}
    		    }
    		}
    	}

    	return $alerts;
    }

    // ADM

    /**
    *** @name: getMoveFoldersList
    *** @desc: Get folders in wich the current user can move a plan or a folder
    *** @param: (STRING) $rootName => name of the plan root folder from $lang $lang['PLAN_FOLDER_ROOT'])
    *** @param: (INTEGER) $parentId => id of the parent folder of the current plan or folder
	*** @param: (INTEGER) $folderId => id of the current element if it's a folder
    *** @retval: (ARRAY) => list of folders wich the user can move the current plan or folder inside
	**/
    public function getMoveFoldersList($rootName='Root', $parentId, $folderId=null)
    {
        $moveList = array();

        // Get the list of folders wich the user has the write privilege on it
        $folders = Model_Plan::getFolders($this->dbconnection, null, Model_plan::PLAN_SORT_PARENT, $this->user->getId(), true);
        if (!empty($folderId))
        {
            // If the element to move is a folder, we must remove it's children from the list. Moving it inside one of it's children will break the tree.
            $folderChildren = Model_Plan::getFolders($this->dbconnection, $folderId, Model_plan::PLAN_SORT_CHILDREN);
            $folders = array_diff_key($folders, $folderChildren);
        }
        if (!empty($parentId) && empty($folders[$parentId]))
        {
            // If parent isn't in the folders write list because the user hasn't the write right on it. In order to keep the current parent, we have to add it into the move list.
            $foldersParent = Model_Plan::getFolders($this->dbconnection, $parentId);
            $folders[$parentId] = $foldersParent[$parentId];
        }

        if (!empty($folders))
        {
            foreach($folders as $fold)
            {
                $id = $fold['id'];
                $name = $folders[$id]['name'];
                $pid = $fold['parent']; // Parent ID
                if (!empty($pid))
                {
                    $pname = '...'; // Parent name
                    if (!empty($folders[$pid]['name']))
                    {
                        $pname = $folders[$pid]['name'];
                    }
                    $name = $pname . ' > ' . $name;
                    if (!empty($folders[$pid]['parent']))
                    {
                        $name = '... > ' . $name;
                    }
                }
                $moveList[] = array('id'=>$id, 'name'=>$name);
            }
            // Sort list by name
            usort($moveList, function($a, $b) {
        		return strcmp($a['name'], $b['name']);
        	});
        }
        // Add root folder to the list
        array_unshift($moveList, array('id' => 0, 'name' => $rootName));

        return $moveList;
    }
    /**
    *** @name: listPictureName
    *** @desc: List the availables background images for plan image
    *** @param: (STRING) $pathFolder => path to the pictures folder
    *** @retval: (ARRAY) => list of background images
	**/
    public function listPictureName($pathFolder)
    {
    	$ext_list = Array("jpg", "JPG", "jpeg", "JPEG", "bmp", "BMP", "gif", "GIF", "png", "PNG"); // Extensions
    	$listephotos = Array();
    	$dossier = opendir($pathFolder);
    	for($i=0; $f = readdir($dossier); $i++)
    	{
    		if(is_file($pathFolder.$f))
    		{
    			if($f!="logoFooter.jpg")
    			{
    				if(in_array(preg_replace("#(.+)\.(.+)#", "$2", $f), $ext_list))
    				{
    					$listephotos[$i] = $f;
    				}
    			}
    		}
    	}
    	closedir($dossier);
    	sort($listephotos);
    	return $listephotos;
    }
    /**
    *** @name: setReadWritePrivilegeList
    *** @desc: Set new read write users id list wich can access to a plan or a folder with safety to not auto remove our own user id from the list
    *** @param: (ARRAY) $currentUsersAccessRight => current READ & WRITE users list ids on plan or folder before update privileges (result from Model_Plan::getPlanUserAccess or Model_Plan::getFolderUserAccess)
    *** @param: (ARRAY) $newUsersReadIds => list of new users ids with READ access (from html form value)
    *** @param: (ARRAY) $newUsersWriteIds => list of new users ids with WRITE access (from html form value)
    *** @retval: (ARRAY) => READ = array list of users read ids on this item, WRITE = array list of users write ids on this item
	**/
    public function setReadWritePrivilegeList($currentUsersAccessRight, $newUsersReadIds, $newUsersWriteIds)
    {
        $accessUsersIds = array(
            'READ' => array(),
            'WRITE' => array()
        );
        // Read
        if(is_array($newUsersReadIds))
        {
            $accessUsersIds['READ'] = $newUsersReadIds;
        }
        elseif(is_numeric($newUsersReadIds))
        {
            $accessUsersIds['READ'][] = $newUsersReadIds;
        }
        if(in_array($this->user->getId(), $currentUsersAccessRight['READ']))
        {
            // Because the user can't see his own name into the users list, we must add it to the new list to not remove privileges.
            $accessUsersIds['READ'][] = $this->user->getId();
        }
        // Write
        if(is_array($newUsersWriteIds))
        {
            $accessUsersIds['WRITE'] = $newUsersWriteIds;
        }
        elseif(is_numeric($newUsersWriteIds))
        {
            $accessUsersIds['WRITE'][] = $newUsersWriteIds;
        }
        if(in_array($this->user->getId(), $currentUsersAccessRight['WRITE']))
        {
            // Because the user can't see his own name into the users list, we must add it to the new list to not remove privileges.
            $accessUsersIds['WRITE'][] = $this->user->getId();
        }

        return $accessUsersIds;
    }

}

?>
