<?php
/**
 * @name Model_Plan
 * @description Classe modèle Plan
 *		Toutes les méthodes d'acces à la BD pour les tables concernant les plans
 * @author Nathan Faes <n.faes@esia-sa.com>
 * @date 30/01/2018
 */
include_once MVCMODEL;
class Model_Plan extends Model_esia
{
	const PLAN_SORT_PARENT = 0;
	const PLAN_SORT_CHILDREN = 1;
	/**@NAME: getPlan
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $plan_id l'identifiant du plan
	 *@OUT: array contenant le ou les plans
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getPlan($dbconnection,$plan_id=null,$user=null,$folder=null,$objects=FALSE)
	{
		$sql = "SELECT *
				FROM plan_plans
				LEFT JOIN plan_types ON type_id = plan_type_id";

		if($objects)
        {
            $sql .= " JOIN plan_objects ON object_on_plan_id = plan_id";
        }
        if(!empty($user))
        {
            $sql .= " JOIN plan_users_access ON plan_access_plan_id = plan_id";
        }
		$params=array();
		if(!empty($plan_id)&& $plan_id!=null)
		{
			$sql .= " WHERE plan_id=:plan_id";
			$params[':plan_id']=$plan_id;
		}
        if(!empty($user) && !empty($plan_id))
        {
            $sql .= " AND plan_access_user_id=:user_id";
            $params[':user_id']=$user;
        }
        elseif(!empty($user))
        {
            $sql .= " WHERE plan_access_user_id=:user_id";
            $params[':user_id']=$user;
        }
		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$plans=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if(!isset($plans[$row->plan_id]['id']))
			{
				$plans[$row->plan_id]['id'] = $row->plan_id;
				$plans[$row->plan_id]['name'] = $row->plan_name;
				$plans[$row->plan_id]['params'] = $row->plan_params;
                $plans[$row->plan_id]['folders'] = $row->plan_folder_id;
				$plans[$row->plan_id]['type_id'] = $row->type_id;
				$plans[$row->plan_id]['type_name'] = $row->type_name;
				$plans[$row->plan_id]['type_controller'] = $row->type_controller;
			}

			if($objects)
			{
				$object = array();
				$object['id'] = $row->object_id;
				$object['name'] = $row->object_name;
				$object['icon_id'] = $row->object_icon_id;
				$object['params'] = $row->object_params;

				$plans[$row->plan_id]['objects'][] = $object;
			}
		}

		return $plans;
	}

	/**@NAME: getObjectsByPlan
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $plan_id l'identifiant du plan
	 *     (optionnel) $type : filtrer sur le type d'objet
	 *@OUT: array des objets que contient le plan référencé
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getObjectsByPlan($dbconnection,$plan_id,$type=null)
	{
		$sql = "SELECT *
				FROM plan_objects
				WHERE object_on_plan_id=:plan_id";

		$params=array();
		$params[':plan_id']=$plan_id;

		if(isset($type) && !empty($type))
		{
			$sql .= " AND object_type_id=:type";
			$params[':type']=$type;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$objects=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$objects[] = array(
				'id' => $row->object_id,
				'name' => $row->object_name,
				'icon_id' => $row->object_icon_id,
				'plan_id' => $row->object_on_plan_id,
				'link' => $row->object_link_plan_id,
				'type' => $row->object_type_id,
				'params' => $row->object_params
			);
		}

		return $objects;
	}

	/**@NAME: getObjectsByLink
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $plan_id l'identifiant du plan lié
	 *@OUT: array des objets que contient le plan référencé
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getObjectsByLink($dbconnection,$plan_id)
	{
		$sql = "SELECT *
				FROM plan_objectsfolder_id,folder_name,folder_parent_id,plan_id,plan_name,plan_params,plan_type_id,type_controller
                  FROM plan_folders
                 JOIN plan_plans ON plan_folder_id=folder_id
				JOIN plan_plans ON object_on_plan_id = plan_id
				WHERE object_link_plan_id=:plan_id";

		$params=array();
		$params[':plan_id']=$plan_id;

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$objects=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$objects[] = array(
					'id' => $row->object_id,
					'name' => $row->object_name,
					'plan_name' => $row->plan_name
			);
		}

		return $objects;
	}

	/**@NAME: getObject
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $id l'identifiant de l'objet
	 *@OUT: array contenant l'objet
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getObject($dbconnection,$id)
	{
		$sql = "SELECT *
				FROM plan_objects
				WHERE object_id=:id";

		$params=array();
		$params[':id']=$id;

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$row=$query->fetch(PDO::FETCH_OBJ);
		$objects = array(
				'id' => $row->object_id,
				'name' => $row->object_name,
				'icon_id' => $row->object_icon_id,
				'plan_id' => $row->object_on_plan_id,
				'link' => $row->object_link_plan_id,
				'type' => $row->object_type_id,
				'params' => $row->object_params
		);

		return $objects;
	}

	/**@NAME: getObject
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $id l'identifiant de l'icon
	 *@OUT: array contenant l'objet
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getObjectbyIcon($dbconnection,$id)
	{
		$sql = "SELECT *
				FROM plan_objects
				WHERE object_icon_id=:id";

		$params=array();
		$params[':id']=$id;

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);


		$objects=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$objects[] =array(
				'id' => $row->object_id,
				'name' => $row->object_name,
				'icon_id' => $row->object_icon_id,
				'plan_id' => $row->object_on_plan_id,
				'link' => $row->object_link_plan_id,
				'type' => $row->object_type_id,
				'params' => $row->object_params
			);
		}

		return $objects;
	}

	/**@NAME: getPlanTypes
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $plan_id l'identifiant du plan
	 *@OUT: array contenant le ou les types de plans
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getPlanType($dbconnection,$plan_id=null)
	{
		$sql = "SELECT *
				FROM plan_types";

		$params=array();
		if(!empty($plan_id) && $plan_id!=null)
		{
			$sql .= " WHERE type_id= (SELECT plan_type_id FROM plan_plans WHERE plan_id=:plan_id)";
			$params[':plan_id']=$plan_id;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$types=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$types[] = array('id'=>$row->type_id,
					'name'=>$row->type_name,
					'controller'=>$row->type_controller
			);
		}

		return $types;
	}

	/**@NAME: getIcon
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $id l'identifiant de l'icone
	 *@OUT: array contenant le ou les icones
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getIcon($dbconnection,$id=null)
	{
		$sql = "SELECT *
				FROM plan_object_icons";

		$params=array();
		if(!empty($id) && $id!=null)
		{
			$sql .= " WHERE icon_id=:id";
			$params[':id']=$id;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$icons=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$icons[] = array('id'=>$row->icon_id,
					'name'=>$row->icon_name,
					'path'=>$row->icon_path,
					'type'=>$row->icon_type,
					'group'=>$row->icon_group_id,
			);
		}

		return $icons;
	}

	/**@NAME: getIconByGroups
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $group l'identifiant du groupe
	 *@OUT: array contenant le ou les groupes contenant les icones
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getIconByGroups($dbconnection,$group=null)
	{
		$sql = "SELECT *
				FROM plan_object_icons
				LEFT JOIN plan_icon_groups ON icon_group_id=group_id";

		$params=array();
		if(!empty($group) && $group!=null)
		{
			$sql .= " WHERE group_id=:id";
			$params[':group']=$group;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$icons=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if($row->group_name)
			{
				$icons[$row->group_name][] = array('id'=>$row->icon_id,
						'name'=>$row->icon_name,
						'path'=>$row->icon_path,
						'type'=>$row->icon_type,
						'group'=>$row->icon_group_id
				);
			}
			else
			{
				$icons['PLAN_NO_GROUP'][] = array('id'=>$row->icon_id,
						'name'=>$row->icon_name,
						'path'=>$row->icon_path,
						'type'=>$row->icon_type,
						'group'=>$row->icon_group_id
				);
			}
		}

		return $icons;
	}

	/**@NAME: getIconGroup
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $id l'identifiant du groupe
	 *@OUT: array contenant le ou les groupes
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getIconGroup($dbconnection,$id=null)
	{
		$sql = "SELECT *
				FROM plan_icon_groups";

		$params=array();
		if(!empty($id) && $id!=null)
		{
			$sql .= " WHERE group_id=:id";
			$params[':id']=$id;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$groups=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$groups[] = array('id'=>$row->group_id,
					'name'=>$row->group_name,
			);
		}

		return $groups;
	}

	/**@NAME: getObjectTypes
	 *@IN: $dbconnection l'objet de connexion PDO
	 *     $id l'identifiant du type
	 *@OUT: array contenant le ou les types d'objet
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public static function getObjectType($dbconnection,$id=null)
	{
		$sql = "SELECT *
				FROM plan_object_types";

		$params=array();
		if(!empty($id) && $id!=null)
		{
			$sql .= " WHERE type_id=:id";
			$params[':id']=$id;
		}

		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$types=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$types[] = array('id'=>$row->type_id,
					'name'=>$row->type_name,
					'controller'=>$row->type_controller,
					'params'=>$row->type_default_params

			);
		}

		return $types;
	}

	/**
	 * @brief addPlan : ajouter un plan
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $name le nom du plan
	 * @param integer $type l'identifiant du type de plan à ajouter
	 * @param json $params paramètres du plan au format JSON
	 * @param (optionnel) array $objects objets à ajouter sur le plan
	 * @param (optionnel) integer $user identifiant de l'utilisateur créateur
	 * @param (optionnel) integer $identifiant du répertoire
	 */
	public static function addPlan($dbconnection,$name,$type,$params=null,$objects=null,$user=null,$folderid=null)
	{
		$query= $dbconnection->prepare("INSERT INTO plan_plans (plan_id,plan_name,plan_params,plan_type_id)
				    					VALUES(DEFAULT,:name,:params,:type) RETURNING plan_id");

		$query = Model_esia::executeStatement($query,array(':name' =>$name,
				':params'=>$params,
				':type'=>$type
		));

		$plan_id = $query->fetch(PDO::FETCH_OBJ)->plan_id;

		if(isset($objects) && !empty($objects))
		{
			foreach($objects as $obj)
			{
				Model_plan::addObjectPlan($dbconnection,$plan_id,$obj['name'],$obj['type'],$obj['icon'],$obj['params']);
			}
		}
		if(!empty($user))
        {
		    $query= $dbconnection->prepare("INSERT INTO plan_users_access(plan_access_user_id, plan_access_plan_id, plan_access_privileges) VALUES (?, ?,'CREATOR');");
            $query = Model_esia::executeStatement($query,array($user,$plan_id));
        }
        if(!empty($folderid))
        {
            $query= $dbconnection->prepare("UPDATE plan_plans SET plan_folder_id=? WHERE plan_id=?;");
            $query = Model_esia::executeStatement($query,array($folderid,$plan_id));

        }
		return $plan_id;
	}

	/**
	 * @brief addObjectPlan : ajouter un objet sur un plan
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $name le nom de l'objet
	 * @param integer $type l'identifiant du type de l'objet à ajouter
	 * @param integer $icon l'identifiant de l'icone de l'objet à ajouter
	 * @param json $params paramètres de l'objet au format JSON
	 * @param integer $link lien du plan que l'objet référence
	 */
	public static function addObjectPlan($dbconnection,$plan,$name,$type,$icon,$params,$link=null)
	{
		$query= $dbconnection->prepare("INSERT INTO plan_objects (object_id,object_name,object_on_plan_id,object_link_plan_id,object_icon_id,object_params,object_type_id)
				    					VALUES(DEFAULT,:name,:plan,:link,:icon,:params,:type) RETURNING object_id");

		$query = Model_esia::executeStatement($query,array(':name' =>$name,
				':plan'=>$plan,
				':link'=>$link,
				':icon'=>$icon,
				':params'=>$params,
				':type'=>$type
		));

		$object = $query->fetch(PDO::FETCH_OBJ)->object_id;

		return $object;
	}

	/**
	 * @brief addIcon : ajouter une icone
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $name le nom de l'icone
	 * @param string $path chemin vers l'image
	 */
	public static function addIcon($dbconnection,$name,$path,$type='svg',$group)
	{
		$query= $dbconnection->prepare("INSERT INTO plan_object_icons (icon_id,icon_name,icon_path,icon_type,icon_group_id)
				    					VALUES(DEFAULT,:name,:path,:type,:group) RETURNING icon_id");

		$query = Model_esia::executeStatement($query,array(':name' =>$name,
				':path'=>$path,
				':type'=>$type,
				':group'=>$group
		));

		$icon = $query->fetch(PDO::FETCH_OBJ)->icon_id;

		return $icon;
	}

	/**
	 * @brief addIconGroup : ajouter un groupe d'icônes
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $name le nom de l'icone
	 */
	public static function addIconGroup($dbconnection,$name)
	{
		$query= $dbconnection->prepare("INSERT INTO plan_icon_groups (group_id,group_name)
				    					VALUES(DEFAULT,:name) RETURNING group_id");

		$query = Model_esia::executeStatement($query,array(':name' =>$name));

		$group = $query->fetch(PDO::FETCH_OBJ)->group_id;

		return $group;
	}

	/**
	 * @brief deletePlan : supprimer un plan
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id du plan à supprimer
	 */
	public static function deletePlan($dbconnection,$id)
	{
	    $query= $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id=:id");
	    $query = Model_esia::executeStatement($query,array(':id' =>$id));
	    $query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_on_plan_id=:plan");
	    $query = Model_esia::executeStatement($query,array(':plan'=>$id));
		$query= $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_id=:id");
		$query = Model_esia::executeStatement($query,array(':id' =>$id));

		$plan = $query->fetch(PDO::FETCH_OBJ)->plan_id;

		return $plan;
	}

	/**
	 * @brief deleteObject : supprimer un objet
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id de l'objet à supprimer
	 */
	public static function deleteObject($dbconnection,$id)
	{
		$query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_id=:id RETURNING object_id");

		$query = Model_esia::executeStatement($query,array(':id' =>$id));

		$object = $query->fetch(PDO::FETCH_OBJ)->object_id;

		return $object;
	}

	/**
	 * @brief deleteObject : supprimer tous les objets d'un plan
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id du plan sur lequel se trouve l'objet
	 */
	public static function deleteAllObjOnPlan($dbconnection,$plan)
	{
		$query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_on_plan_id=:plan");

		$query = Model_esia::executeStatement($query,array(':plan'=>$plan));
	}

	/**
	 * @brief deleteObject : supprimer un objet
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id de l'objet à supprimer
	 * @param integer $id du plan sur lequel se trouve l'objet
	 */
	public static function deleteObjOnPlan($dbconnection,$id,$plan)
	{
		$query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_id=:id AND object_on_plan_id=:plan RETURNING object_name");

		$query = Model_esia::executeStatement($query,array(':id' =>$id,':plan'=>$plan));

		$object = $query->fetch(PDO::FETCH_OBJ)->object_name;

		return $object;
	}

	/**
	 * @brief deleteObject : supprimer un objet lien
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param string $typeIdName partie du nom de l'objet à supprimer
	 * @param integer $plan du plan sur lequel se trouve l'objet
	 */
	public static function deleteObjLinkOnPlan($dbconnection,$typeIdName,$plan)
	{
		$query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_name ILIKE :name AND object_on_plan_id=:plan AND object_type_id=1");

		$query = Model_esia::executeStatement($query,array(':name' =>'%'.$typeIdName.'%',':plan'=>$plan));
	}

	/**
	 * @brief deleteIcon : supprimer une icone
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id de l'icone à supprimer
	 */
	public static function deleteIcon($dbconnection,$id)
	{
		$query= $dbconnection->prepare("DELETE FROM plan_object_icons WHERE icon_id=:id RETURNING icon_id");

		$query = Model_esia::executeStatement($query,array(':id' =>$id));

		$icon = $query->fetch(PDO::FETCH_OBJ)->icon_id;

		return $icon;
	}

	/**
	 * @brief deleteIconGroup : supprimer un groupe d'icônes
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $group l'id du groupe
	 */
	public static function deleteIconGroup($dbconnection,$group)
	{
		$query = $dbconnection->prepare("UPDATE plan_object_icons SET icon_group_id=DEFAULT WHERE icon_group_id=:group");
		$query = Model_esia::executeStatement($query,array(':group' =>$group));

		$query = $dbconnection->prepare("DELETE FROM plan_icon_groups WHERE group_id=:group RETURNING group_id");
		$query = Model_esia::executeStatement($query,array(':group' =>$group));
		$group = $query->fetch(PDO::FETCH_OBJ)->group_id;

		return $group;
	}

	/**
	 * @brief updatePlan : update un plan
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id identifiant du plan à modifier
	 * @param string $name le nom du plan
	 * @param integer $type l'identifiant du type de plan
	 * @param integer $folderid l'identifiant du nouveau répertoire
	 * @param json $params paramètres du plan au format JSON
	 */
	public static function updatePlan($dbconnection,$id,$name,$type,$params,$folderid=null)
	{
		$query= $dbconnection->prepare("UPDATE plan_plans SET plan_name=:name , plan_type_id=:type , plan_params=:params
										WHERE plan_id = :id
										RETURNING plan_id;");

		$query = Model_esia::executeStatement($query,array(':id' =>$id,
				':name' =>$name,
				':type'=>$type,
				':params'=>$params));
		$row=$query->fetch(PDO::FETCH_OBJ);
		if(is_numeric($folderid))
		{
			$sqlParam = array(':id' =>$id);
			if($folderid == 0)
			{
				$sql = "UPDATE plan_plans SET plan_folder_id=NULL WHERE plan_id = :id;";
			}
			else
			{
				$sql = "UPDATE plan_plans SET plan_folder_id=:folder WHERE plan_id = :id;";
				$sqlParam['folder'] = $folderid;
			}
			$query= $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query,$sqlParam);
		}
		return $row->plan_id;
	}

	/**
	 * @brief updateObjLinkedPlan : update les références plan d'un object
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id identifiant du plan référencé
	 */
	public static function updateObjLinkedPlan($dbconnection,$idPlan)
	{
		$query= $dbconnection->prepare("UPDATE plan_objects SET object_link_plan_id=null
										WHERE object_link_plan_id = :idPlan
										RETURNING object_id;");

		$query = Model_esia::executeStatement($query,array(':idPlan' =>$idPlan));
		$object=$query->fetch(PDO::FETCH_OBJ)->object_id;
		return $object;
	}

	/**
	 * @brief updateObject : modifier un objet
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id identifiant de l'objet à modifier
	 * @param string $name le nom de l'objet
	 * @param integer $type l'identifiant du type de l'objet
	 * @param integer $icon l'identifiant de l'icone de l'objet
	 * @param json $params paramètres de l'objet au format JSON
	 * @param integer $link lien du plan que l'objet référence
	 */
	public static function updateObject($dbconnection,$id,$plan,$name,$type,$icon,$params,$link)
	{
		$query= $dbconnection->prepare("UPDATE plan_objects SET object_name=:name,object_on_plan_id=:plan,object_link_plan_id=:link,object_icon_id=:icon,object_params=:params,object_type_id=:type
										WHERE object_id=:id
										RETURNING object_id");

		$query = Model_esia::executeStatement($query,array(':id' =>$id,
				':name' =>$name,
				':plan'=>$plan,
				':link'=>$link,
				':icon'=>$icon,
				':params'=>$params,
				':type'=>$type
		));

		$object = $query->fetch(PDO::FETCH_OBJ)->object_id;

		return $object;
	}

	/**
	 * @brief updateIcon : modifier une icone
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id identifiant de l'icone à modifier
	 * @param string $name le nom de l'icone
	 * @param string $path chemin vers l'image
	 */
	public static function updateIcon($dbconnection,$id,$name,$path,$type='svg',$group)
	{
		$query= $dbconnection->prepare("UPDATE plan_object_icons SET icon_name=:name,icon_path=:path,icon_type=:type, icon_group_id=:group
										WHERE icon_id = :id
										RETURNING icon_id");

		$query = Model_esia::executeStatement($query,array(':id' =>$id,
				':name' =>$name,
				':path'=>$path,
				':type'=>$type,
				':group'=>$group
		));

		$icon = $query->fetch(PDO::FETCH_OBJ)->icon_id;

		return $icon;
	}

	/**
	 * @brief updateIconGroup : modifier un groupe d'icônes
	 * @param PDO $dbconnection  l'objet de connexion PDO
	 * @param integer $id identifiant du groupe à modifier
	 * @param string $name le nom du groupe
	 */
	public static function updateIconGroup($dbconnection,$id,$name)
	{
		$query= $dbconnection->prepare("UPDATE plan_icon_groups SET group_name=:name
										WHERE group_id = :id
										RETURNING group_id");

		$query = Model_esia::executeStatement($query,array(':id' =>$id,
				':name' =>$name,
		));

		$group = $query->fetch(PDO::FETCH_OBJ)->group_id;

		return $group;
	}

	/*@NAME: GetMapConfiguration
	 *@IN: $dbconnection l'objet de connexion PDO
	 *@OUT: un tableau associatif les clés sont:
	 *  $tab['CENTER_LATITUDE']: la latitude du centre de la carte par défaut
	 *  $tab['CENTER_LONGITUDE']: la longitude du centre de la carte par défaut
	 *  $tab['TYPE']: le type de la carte par défaut
	 *  $tab['ZOOM']: le niveau de zoom par défaut
	 *  $tab['ZOOM_CHANGED']: le niveau de zoom du changement bâtiment/zone par défaut
	 *  $tab['IMAGE_MAX_SIZE']: taille maximun d'une image en Mo
	 *  $tab['IMG_DIRECTORY']: repertoire de destination des images (apache)
	 *@TREAT:  Effectue une requête sur la base de donnée pour récupérer les valeurs par défaut de la carte. (situé dans esia_configuration => clé MAP_*)
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public function GetMapConfiguration($dbconnection)
	{
		$conf = array();
		$query = $dbconnection->prepare("SELECT config_key,config_value
				FROM esia_configuration
				WHERE config_key LIKE 'MAP_%'");
		$query = Model_esia::executeStatement($query);
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if($row->config_key=='MAP_CENTER_LATITUDE')
				$conf['CENTER_LATITUDE'] = $row->config_value;
				elseif($row->config_key=='MAP_CENTER_LONGITUDE')
				$conf['CENTER_LONGITUDE'] = $row->config_value;
				elseif($row->config_key=='MAP_TYPE')
				$conf['TYPE'] = $row->config_value;
				elseif($row->config_key=='MAP_ZOOM')
				$conf['ZOOM'] = $row->config_value;
				elseif($row->config_key=='MAP_ZOOM_CHANGED')
				$conf['ZOOM_CHANGED'] = $row->config_value;
				elseif($row->config_key=='MAP_IMG_TAILLE_MAX')
				$conf['IMAGE_MAX_SIZE'] = $row->config_value;
				elseif($row->config_key=='MAP_IMG_DIR')
				$conf['IMG_DIRECTORY'] = $row->config_value;
		}
		return $conf;
	}
	/*@NAME: getStatePlan
	 *@IN: $dbconnection l'objet de connexion PDO
	 *	$id: identifiant du plan
	 *	$userid: identifiant de l'utilisateur
	 *	$lvl: (default: 0) niveau de récursivité actuelle
	 *	$maxlvl: (default: 4) nombre maximun d'appelle de fonction autorisé en récusif
	 *@OUT: un champ texte avec le statut du plan (ok,warning,critical,unknown et pending)
	 *@TREAT:  Retourne l'état d'un plan donnée
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public function getStatePlan($dbconnection,$id,$userid,$lvl=0,$maxlvl=4 )
	{
		$objects = Model_plan::getObjectsByPlan($dbconnection,$id);
		$globalstate = 'ok';
		if($lvl < $maxlvl)
		{
			$nodes = array();
			$groups = array();
			$plans=array();
			$nodes = array_map('intval',explode(",",$param['nodes']));
			foreach($objects as $key => $object)
			{
				$param= json_decode($object['params'],true);
				if($param['getstate']!='no')
				{
    				if(array_key_exists('nodes', $param))
    				{
    					$nodes = array_merge($nodes,explode(",",$param['nodes']));
    				}
    				if(array_key_exists('groups', $param))
    				{
    					$groups= array_merge($groups,explode(",",$param['groups']));
    				}
    				if(!empty($object['link']))
    				{
    					$plans[]= $object['link'];
    				}
				}
			}
			//on commence par l'état des noeuds
			$nodes = array_map('intval',$nodes);
			$listState = Model_esia::countNodeState($dbconnection,$nodes);
			$globalstate = Model_plan::getMostImportantStatus($globalstate,$listState);
			if($globalstate =='critical')
			{
				return 'critical';
			}
			//tous les noeuds sont OK on va vérifier l'état des groups
			$groups = array_map('intval',$groups);
			foreach($groups as $group)
			{
				$state_parent =  model_esia::getGroupNodeStates($dbconnection,$userid,$group,false);
				$globalstate = Model_plan::getMostImportantStatus($globalstate,$state_parent[$group]);
				if($globalstate =='critical')
				{
					return 'critical';
				}
				else
				{
					$state_child =  model_esia::getGroupNodeStates($dbconnection,$userid,$group,true);
					foreach($state_child as $current)
					{
						$globalstate = Model_plan::getMostImportantStatus($globalstate,$current);
						if($globalstate =='critical')
						{
							return 'critical';
						}
					}
				}
			}
			//rien dans les groupes n'est en critique on passe au plan
			foreach($plans as $plan)
			{
				$stateplan = Model_Plan::getStatePlan($dbconnection, $plan,$userid,$lvl+1,$maxlvl);
				$globalstate = Model_plan::getMostImportantStatus($globalstate,$stateplan);
				if($globalstate =='critical')
				{
					return 'critical';
				}
			}
		}
		return $globalstate;
	}
	/*@NAME: getObjectState
	 *@IN: $dbconnection l'objet de connexion PDO
	 *	$object: un tableau associatif (voir model_plan::getObjectsByPlan )
	 *	$userid: identifiant de l'utilisateur
	 *	$lvl: (default: 0) niveau de récursivité actuelle
	 *	$maxlvl: (default: 4) nombre maximun d'appelle de fonction autorisé en récusif
	 *@OUT: un champ texte avec le statut de l'objet (ok,warning,critical,unknown et pending)
	 *@TREAT:  Retourne l'état d'un objet passé en paramètre
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	public function getObjectState($dbconnection,$object,$userid,$lvl=0,$maxlvl=4)
	{
		$globalstate = 'ok';
		$param = $object['params'];
		if(array_key_exists('nodes', $param))
		{
			$nodes = array_map('intval',explode(",",$param['nodes']));
			$listState = Model_esia::countNodeState($dbconnection,$nodes);
			$globalstate = Model_plan::getMostImportantStatus($globalstate,$listState);
			if($globalstate =='critical')
			{
				return 'critical';
			}
		}
		if(array_key_exists('groups', $param))
		{
			$groups = array_map('intval',explode(",",$param['groups']));

			foreach($groups as $group)
			{
				$state_parent =  model_esia::getGroupNodeStates($dbconnection,$userid,$group,false);
				$globalstate = Model_plan::getMostImportantStatus($globalstate,$state_parent[$group]);
				if($globalstate =='critical')
				{
					return 'critical';
				}
				else
				{
					$state_child =  model_esia::getGroupNodeStates($dbconnection,$userid,$group,true);
					foreach($state_child as $current)
					{
						$globalstate = Model_plan::getMostImportantStatus($globalstate,$current);
						if($globalstate =='critical')
						{
							return 'critical';
						}
					}
				}
			}
		}
		if(!empty($object['link']))
		{
			$stateplan = Model_Plan::getStatePlan($dbconnection, $object['link'],$userid,$lvl+1,$maxlvl);
			$globalstate = Model_plan::getMostImportantStatus($globalstate,$stateplan);
		}
		return $globalstate;
	}
	/*@NAME: getMostImportantStatus
	 *@IN: $dbconnection l'objet de connexion PDO
	 *	$globalState: le status actuelle sous forme de chaine de caractère (ok,warning,critical,pending,unknown)
	 *	$array: l'état que le souhaite tester sous forme de tableau associatif (voir getGroupNodeStates) ou sous forme de chaine de caractère
	 *@OUT: un champ texte avec le statut le plus critique
	 *@TREAT:  Fait une comparaison entre les 2 états
	 *@EXCEPTION: 	EsiaDbException si une erreur survient
	 */
	private function getMostImportantStatus($globalState,$array)
	{
		if(is_string($array))
		{
			if($array=='critical' )
			{
				$globalState='critical';
			}
			elseif($array=='warning' && $globalState!='critical')
			{
				$globalState='warning';
			}
			elseif($array=='unknown' && $globalState!='critical' && $globalState!='warning')
			{
				$globalState='unknown';
			}
			elseif($array=='OK') $globalState='ok';
			elseif($array=='pending') $globalState='pending';
		}
		else
		{
			if($array['critical']> 0 )
			{
				$globalState='critical';
			}
			elseif($array['warning']> 0 && $globalState!='critical')
			{
				$globalState='warning';
			}
			elseif($array['unknown']> 0 && $globalState!='critical' && $globalState!='warning')
			{
				$globalState='unknown';
			}
			elseif($array['OK']> 0) $globalState='ok';
			elseif($array['pending']> 0) $globalState='pending';
		}
		return $globalState;
	}
	/**
	 * Récupére un dossier ou la liste des dossiers
	 * @param PDO $dbconnection un objet PDO
	 * @param array  $id valeur par défaut null, un integer ou un tableau d'integer. Les integers correspondent à l'identifiant DB
	 * @throws EsiaDbException
	 * @retval array un tableau associatif contenant les clés suivante
     *      'id'=>$row->folder_id           identifiant db du plan
     *      'name'=>$row->folder_name,      nom du plan
     *      'parent'=>$row->folder_parent_id identifiant du parent si NULL alors il s'agit d'un dossier racine
     *      'plan'=>array()); // tableau d'array des plan (nom implémenté actuellement)
	 */
	public static function getFolders($dbconnection,$id=null,$tree_direction=Model_Plan::PLAN_SORT_PARENT,$user_id=null,$onlyWrite=false)
    {
        $sql="SELECT folder_id,folder_name,folder_parent_id
              FROM plan_folders";
        $param=array();
		$db_function = 'getallParentPlanFolder';
		if($tree_direction == Model_Plan::PLAN_SORT_CHILDREN)
		{
			$db_function = 'getallchildplanfolder';
		}
        if(is_numeric($id))
        {
			$sql=" SELECT fold_id as folder_id,fold_name as folder_name,fold_parent_id as folder_parent_id FROM ".$db_function."(:id::integer)";
            $param[':id']=$id;
        }
        elseif(is_array($id))
        {
            foreach($id as $folder)
            {
                if(!is_numeric($folder))
                        throw new EsiaDbException('$folder array must be numeric');
            }
            $arrids = implode(',', $id);
			$sql=" SELECT fold_id as folder_id,fold_name as folder_name,fold_parent_id as folder_parent_id FROM ".$db_function."('{ $arrids }'::int[])";
        }
        if(is_numeric($user_id))
        {
            $sql.= " JOIN plan_plans ON plan_folder_id =folder_id
                    JOIN plan_users_access ON plan_access_plan_id= plan_id
                    WHERE plan_name='.ACCESS' AND plan_access_user_id = :userid";
            $param[':userid']=$user_id;
			if($onlyWrite)
			{
				$sql.= " AND (plan_access_privileges = 'CREATOR' OR plan_access_privileges = 'WRITE')";
			}
        }
        $sql.=" ORDER BY folder_parent_id DESC,folder_name;";
		$query= $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$param);
        $folders=array();
	    while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            if(!array_key_exists($row->folder_id,$folders))
            {
                $folders[$row->folder_id]=array('id'=>$row->folder_id,'name'=>$row->folder_name,'parent'=>$row->folder_parent_id,'plan'=>array());
            }
        }
		return $folders;
    }
    /**
     * met à jour les droits d'accès à un plan
     * @param PDO $dbconnection un objet PDO
     * @param integer  $planid identifiant du plan
     * @param array  $userids tableau d'integer contenant les identifiants des l'utilisateurs
     * @param string  [READ|WRITE] niveau de droit d'accès
     * @throws EsiaDbException
     * @retval
     */
    public static function updatePlanPrivilege($dbconnection,$planid,$userids,$right)
    {
        $query= $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id=? AND plan_access_privileges = ?");
        $query = Model_esia::executeStatement($query,array($planid,$right));

        $query= $dbconnection->prepare("INSERT INTO plan_users_access(plan_access_user_id, plan_access_plan_id, plan_access_privileges)VALUES (?, ?, ?);");
        foreach($userids as $id)
        {
            $query = Model_esia::executeStatement($query,array($id,$planid,$right));
        }
    }
	/**
     * retourne les droit d'accès d'un plan pour un utilisateur
     * @param PDO $dbconnection un objet PDO
     * @param integer $planId identifiant du dossier/répertoire
     * @param integer $userId identifiant de l'utilisateur à tester
     * @throws EsiaDbException
     * @retval string [NONE|READ|WRITE|CREATOR]
     */
	public static function getPlanPrivilege($dbconnection,$planId,$userId)
    {
        $access = 'NONE';
		$result = array();
		$privPriority = array('CREATOR', 'WRITE', 'READ');
        $query = $dbconnection->prepare("SELECT plan_access_privileges
                                        FROM plan_plans
                                        JOIN plan_users_access ON plan_id = plan_access_plan_id
										WHERE plan_name!='.ACCESS' AND plan_id=? AND plan_access_user_id=?");

        $query = Model_esia::executeStatement($query,array($planId,$userId));
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
            array_push($result, $row->plan_access_privileges);
        }
		if(!empty($result))
		{
			$result = array_intersect($privPriority, $result);
			$access = array_shift($result);
		}
        return $access;
    }
    /**
     * retourne les utilisateurs et leurs droits d'accès accès à un plan donné
     * @param PDO $dbconnection un objet PDO
     * @param integer  $planId identifiant du plan
     * @throws EsiaDbException
     * @retval string [NONE|READ|WRITE|CREATOR]
     */
	public static function getPlanUserAccess($dbconnection,$planId)
    {
        $query = $dbconnection->prepare("SELECT plan_access_privileges,user_id,user_name
                                        FROM plan_plans
										JOIN plan_users_access ON plan_id = plan_access_plan_id
										JOIN esia_users ON plan_access_user_id = user_id
										WHERE plan_name!='.ACCESS' AND plan_id=?");

        $query = Model_esia::executeStatement($query,array($planId));
        $arr['WRITE'] = array();
        $arr['READ'] = array();
        $arr['CREATOR'] = array();
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
			array_push($arr[$row->plan_access_privileges],$row->user_id);
        }
        return $arr;
    }
    /**
     * Ajoute un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param string  $foldername nom du dossier
     * @param integer $parent identifiant du dossier parents
     * @param integer $creator identifiant de l'utilisateur qui a crée le dossier
     * @throws EsiaDbException
     * @retval integer $folderid identifiant du nouveau dossier/répertoire
     */
    public static function addFolder($dbconnection,$foldername,$parent=null,$creator=null)
    {
        $param = array(':name'=>$foldername);
        if(!empty($parent))
        {
            $sql = "INSERT INTO plan_folders(folder_id, folder_name, folder_parent_id)VALUES (DEFAULT, :name, :parent) RETURNING folder_id;";
            $param['parent']=$parent;
        }
        else
        {
            $sql = "INSERT INTO plan_folders(folder_id, folder_name)VALUES (DEFAULT, :name) RETURNING folder_id;";
        }
        $query= $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$param);
        $row=$query->fetch(PDO::FETCH_OBJ);
        $folderid = $row->folder_id;
        $sql = "INSERT INTO plan_plans(plan_id, plan_name, plan_folder_id)VALUES (DEFAULT,'.ACCESS', ?) RETURNING plan_id;";
        $query= $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,array($row->folder_id));
        $row=$query->fetch(PDO::FETCH_OBJ);
        model_plan::updatePlanPrivilege($dbconnection, $row->plan_id, array($creator), 'CREATOR');
        return $folderid;
    }
    /**
     * Modifie un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param integer $folderid identifiant du dossier/répertoire
     * @param string  $foldername nouveau nom du dossier
     * @param integer $parent identifiant du dossier parents
     * @throws EsiaDbException
     * @retval
     */
    public static function updateFolder($dbconnection,$folderid, $foldername,$parent=null)
    {
        $param = array(':name'=>$foldername,':id'=>$folderid);
        if(isset($parent))
        {
			if($parent == 0)
			{
				$sql = "UPDATE plan_folders SET folder_name = :name, folder_parent_id =NULL WHERE folder_id = :id";
			}
			else
			{
				$sql = "UPDATE plan_folders SET folder_name = :name, folder_parent_id = :parent WHERE folder_id = :id";
	            $param['parent']=$parent;
			}
        }
        else
        {
            $sql = "UPDATE plan_folders SET folder_name = :name WHERE folder_id = :id";
        }
        $query= $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$param);
    }
    /**
     * Modifie les droit d'accès un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param integer $folderid identifiant du dossier/répertoire
     * @param array $creator tableau d'identifiant utilisateur qui auront les droits "créateur"
     * @param array $userid_read tableau d'identifiant utilisateur qui auront les droits en lecture
     * @param array $userid_write tableau d'identifiant utilisateur qui auront les droits en écriture
     * @throws EsiaDbException
     * @retval
     */
	public static function updateFolderPrivilege($dbconnection,$folderid,$creator=false,$userid_read=false,$userid_write=false)
    {
        $query= $dbconnection->prepare("SELECT plan_id FROM plan_plans WHERE plan_folder_id=?");
        $query = Model_esia::executeStatement($query,array($folderid));
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
            if($creator !== false)
            {
                model_plan::updatePlanPrivilege($dbconnection, $row->plan_id, $creator, 'CREATOR');
            }
            if($userid_read !== false)
            {
                model_plan::updatePlanPrivilege($dbconnection, $row->plan_id, $userid_read, 'READ');
            }
            if($userid_write !== false)
            {
                model_plan::updatePlanPrivilege($dbconnection, $row->plan_id, $userid_write, 'WRITE');
            }   
        }
        if($userid_write !== false)
        {
            model_plan::updatePlanPrivilege($dbconnection, $row->plan_id, $userid_write, 'WRITE');
        }
    }
    /**
     * retourne les droit d'accès d'un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param integer $folderid identifiant du dossier/répertoire
     * @param integer $user_id identifiant de l'utilisateur à tester
     * @throws EsiaDbException
     * @retval string [NONE|READ|WRITE|CREATOR]
     */
	public static function getFolderPrivilege($dbconnection,$folderid,$user_id)
    {
        $access = 'NONE';
		$result = array();
		$privPriority = array('CREATOR', 'WRITE', 'READ');
        $query = $dbconnection->prepare("SELECT plan_access_privileges
                                        FROM plan_plans
                                        JOIN plan_users_access ON plan_id = plan_access_plan_id
                                         WHERE plan_name='.ACCESS' AND plan_folder_id=? AND plan_access_user_id=?");

        $query = Model_esia::executeStatement($query,array($folderid,$user_id));
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
            array_push($result, $row->plan_access_privileges);
        }
		if(!empty($result))
		{
			$result = array_intersect($privPriority, $result);
			$access = array_shift($result);
		}
        return $access;
    }
    /**
     * retourne les droit d'accès d'un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param integer $folderid identifiant du dossier/répertoire
     * @param integer $user_id identifiant de l'utilisateur à tester
     * @throws EsiaDbException
     * @retval string [NONE|READ|WRITE|CREATOR]
     */
    public static function getFolderUserAccess($dbconnection,$folderid)
    {
        $query = $dbconnection->prepare("SELECT plan_access_privileges,user_id,user_name
                                        FROM plan_plans
                                        JOIN plan_users_access ON plan_id = plan_access_plan_id
                                        JOIN esia_users ON plan_access_user_id = user_id
                                         WHERE plan_name='.ACCESS' AND plan_folder_id=?");

        $query = Model_esia::executeStatement($query,array($folderid));
        $arr['WRITE'] = array();
        $arr['READ'] = array();
        $arr['CREATOR'] = array();
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
			array_push($arr[$row->plan_access_privileges],$row->user_id);
        }
        return $arr;
    }
    /**
     * Déplace un plan dans un répertoire/dossier
     * @param PDO $dbconnection un objet PDO
     * @param integer $plan identifiant du plan
     * @param integer $folder identifiant du dossier/répertoire
     * @throws EsiaDbException si une erreur survient
     * @retval NULL
     */
    public static function movePlan($dbconnection,$plan,$folder)
    {
        if(is_numeric($folder) && is_numeric($plan) )
        {
            $query= $dbconnection->prepare("UPDATE plan_plans SET plan_folder_id=:folder WHERE plan_id = :id;");
            $query = Model_esia::executeStatement($query,array(':id' =>$plan,':folder'=>$folder));
        }
        else
        {
            throw new EsiaDbException('$folder and $plan must be numeric');
        }
        return $row->plan_id;
    }
    /**
     * Supprime un répertoire/dossier et les sous élément si spécifié
     * @param PDO $dbconnection un objet PDO
     * @param integer $folderid identifiant du dossier/répertoire
     * @param boolean $deletecontent default(false) si à true supprime également tout le contenut répertoire et plan en dessous
     * @throws EsiaDbException si une erreur survient
     * @retval NULL
     */
    public static function deleteFolder($dbconnection,$folderid,$deletecontent=false)
    {
        if(is_numeric($folderid))
        {
            if($deletecontent == true)
            {
                $query = $dbconnection->prepare("SELECT fold_id,fold_name,fold_parent_id FROM getallchildplanfolder(?::integer);");
                $query = Model_esia::executeStatement($query,array($folderid));
                $list = array();
                while($row=$query->fetch(PDO::FETCH_OBJ))
                {
                    $list[] = $row->fold_id;
                }
                $list = implode(',', $list);
                $sql = "SELECT plan_id FROM plan_plans WHERE plan_folder_id IN (".$list.")";
                $query= $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id IN ($sql)");
                $query = Model_esia::executeStatement($query);
                $query= $dbconnection->prepare("DELETE FROM plan_objects WHERE object_on_plan_id IN ($sql)");
                $query = Model_esia::executeStatement($query);
                $query= $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_folder_id IN (".$list.")");
                $query = Model_esia::executeStatement($query);
                $query= $dbconnection->prepare("DELETE FROM plan_folders WHERE folder_id IN (".$list.")");
                $query = Model_esia::executeStatement($query);
            }
            else
            {
                $query= $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id IN(
                                                 SELECT plan_id FROM plan_plans WHERE plan_folder_id=? AND plan_name='.ACCESS')");
                $query = Model_esia::executeStatement($query,array($folderid));
                $query= $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_folder_id=? AND plan_name='.ACCESS';");
                $query = Model_esia::executeStatement($query,array($folderid));
                $query= $dbconnection->prepare("UPDATE plan_plans SET plan_folder_id=NULL WHERE plan_folder_id=?");
                $query = Model_esia::executeStatement($query,array($folderid));
                $query= $dbconnection->prepare("UPDATE plan_folders SET folder_parent_id=NULL WHERE folder_parent_id=?");
                $query = Model_esia::executeStatement($query,array($folderid));
                $query= $dbconnection->prepare("DELETE FROM plan_folders WHERE folder_id=?");
                $query = Model_esia::executeStatement($query,array($folderid));
            }

        }
        else
        {
            throw new EsiaDbException('$folderid must be numeric');
        }
    }
}
?>
