/*  @name: plan_folder.js
**
**  @description: JavaScript utility functions for plan folders pages
**  @author: Quentin George <q.george@esia-sa.com>
**  @date: 13/06/2019
*/


// Apres l'envoi d'un formulaire, remplace l'entrée de l'historique du navigateur pour empecher le renvoi du formulaire si l'utilisateur recharge sa page. On ne garde que le chemin vers le plugin et l'id du folder
function cleanURI() {
    var splitedUrl = window.location.href.split("?"),
        url = splitedUrl[0],
        uriIdMatched = splitedUrl[1].match(/id=[0-9]+/g);

    if (uriIdMatched !== null) {
        url = url + '?' + uriIdMatched[0];
    }
    history.replaceState(null, null, url);
}

//Vérifie si le menu popup d'ajout d'un plan ne dépasse pas de la fenètre. Se base sur la position du bouton toggler car contrairement a celles du formulaire elles ne varient pas.
function setAddPopUpSafePos(toggler, popup, container) {
    if ((toggler.offset().left - 60) < (container.offset().left + 10)) {
        //A gauche (-60 = initial popup pos left, +10 = container padding left)
        popup.css("left", "0px");
    } else if ((toggler.offset().left + popup.width() - 60) > (container.offset().left + container.width() - 20)) {
        //A droite (-20 = padding left & right)
        popup.css({
            "left": "auto",
            "right": "0px"
        });
    } else {
        //Réinitialise les valeurs
        popup.css({
            "left": "",
            "right": ""
        });
    }

    if ((toggler.offset().top + toggler.height() + popup.height()) > (container.offset().top + container.height() - 20)) {
        //En bas (-20 = padding top & bottom)
        popup.css({
            "top": "auto",
            "bottom": "65px"
        });
    } else {
        //Réinitialise les valeurs
        popup.css({
            "top": "",
            "bottom": ""
        });
    }
}

// Drag & Drop pour le déplacement des plans et dossiers
function planDragDrop() {
    // éléments déplacables
    $(".plan-folder, .plan-file").draggable({
        revert: "invalid",
        containment: ".plan-folder-background",
        helper: "clone",
        cursor: "move",
        start: function() {
            $(this).addClass("dragging-source");
        },
        stop: function() {
            $(this).removeClass("dragging-source");
        }
    });

    // éléments de destination
    $(".plan-folder").droppable({
        accept: ".plan-elem",
        hoverClass: "droppable-hover",
        activeClass: "droppable-active",
        drop: function(event, ui) {
            var item = ui.draggable,
                cible = $(this),
                id = 0,
                name = $(".plan-name", item).text(),
                parentId = 0,
                type = "",
                itemHref = $("a", item).attr("href"),
                cibleHref = $("a", cible).attr("href"),
                regexpFolder = /folder_control.php/,
                regexpID = /id=([0-9]+)/;

            //on récupère les infos nécessaires dans les href des liens du dossier cible et de l'élément a déplacé
            if (!regexpFolder.test(cibleHref)) {
                return 'ERROR: Target is not a folder';
            }
            if (regexpFolder.test(itemHref)) {
                // déterminer si l'élément a déplacer est un plan ou un dossier
                type = "folder";
            } else {
                type = "plan";
            }
            if (regexpID.test(itemHref)) {
                //id de l'élément a déplacer
                id = itemHref.match(regexpID)[1];
            }
            if (regexpID.test(cibleHref)) {
                //id du dossier cible
                parentId = cibleHref.match(regexpID)[1];
            }

            //on envoie les infos au controlleur php via une requète ajax
            $.ajax({
                type: "POST",
                url: "/plugin/plan/controllers/folder_control.php",
                data: "action=move" + "&id=" + id + "&name=" + name + "&parent=" + parentId + "&type=" + type,
                dataType: "json"
            })
            .done(function(data) {
                var html = '';

                //on récupère le message du controlleur php
                if (data.success) {
                    html = '<p class="EventOK messageUtilisateurs">' + data.message + '</p>';
                    //supprimer l'élément déplacé
                    $(item).fadeOut(500, function() {
                        $(this).remove();
                    });
                } else {
                    html = '<p class="EventCritical messageUtilisateurs">' + data.message + '</p>';
                }
                //supprimer les anciens messages
                $(".plan-folder-page #messages").children().remove();
                //afficher le nouveau message pendant 5s
                $(".plan-folder-page #messages").append(html).children().delay(5000).fadeOut(500);
            });
        }
    });
}
