
// Display error or success messages
function displayToolMessage(message, isError=true) {
    var container = $("<div>", {"class": "messageUtilisateurs", "html": "<p>" + message + "</p>"});

    if (isError) {
        container.addClass("EventCritical");
    } else {
        container.addClass("EventOK");
    }

    // Display new message during 5s
    $("#messages").append(container);
    container.delay(5000).fadeOut(500);
}

// Drag object on plan
function dragPlanToolObj(object)
{
	object.draggable({
        containment:$('#plan'),
		stop: function(e, ui) {
			var objId = $(this).attr('data-id'),
                y = ui.position.top,
                x = ui.position.left;

			$.getJSON("/plugin/plan/controllers/object_control.php?action=update&type=redrag&object=" + objId + "&left=" + x + "&top=" + y, function(data) {
				if(data) // Error message
				{
					displayToolMessage(data, true);
				}
			});
		},
	});
}

// Prevent to create an object outside of the plan limits
function checkObjectOnPlanLimits(planWidth, planHeight, objectWidth, objectHeight, position) {
    var maxRight = position.x + objectWidth,
        maxBottom = position.y + objectHeight;

    if (position.x < 4) { // Left
        position.x = 4;
    } else if (maxRight > planWidth) { // Right
        position.x = position.x - ((maxRight - planWidth) + 18);
    }
    if (position.y < 4) { // Top
        position.y = 4;
    } else if (maxBottom > planHeight) { // Bottom
        position.y = position.y - ((maxBottom - planHeight) + 18);
    }

    return position;
}

// Set the tool object on the plan by calling the right function for each object type
function setToolObjectOnPlan(obj, planId="", isAdm=false) {
    var plan = $("#plan" + planId);

    if (!plan[0]) {
        // If no plan match with planId, set plan as generic selector #plan (esemple: for plan img admin mode)
        plan = $("#plan");
    }

    if (obj.type == 5) {
        // Object type Text
        if (isAdm) {
            adm_setObjectTextOnPlan(plan, obj);
        } else {
            setObjectTextOnPlan(plan, obj);
        }
    } else {
        return "Object type ID " + obj.type + " is not a tool object";
    }
}

// Get click position on a plan without triggering any other event or action on plan elements. Must be use with .then to do next actions only when click event has been sended
// initialEvent => the start event wich will required a click position at next click
// plan => the plan where to fetch the click position
// return an object with y & x positons values
// Ex:
// getClickPositionOnPlan(e, $("#plan")).then(function(position) {
//     console.log(position.y);
//     console.log(position.x);
// });
function getClickPositionOnPlan(initialEvent, plan) {
    return new Promise(function(resolve, reject) {
        // Initialise new DOM element #click-capture-box at the same position & size than the plan & above it to capture the click position without triggering any other event or action on DOM element
        var captureBox = $("<div>", {"id": "click-capture-box"});

        if (!$("#click-capture-box").length) {
            // Create #click-capture-box if it doesn't exist yet
            plan.prepend(captureBox);
        }

        // Prevent the event click.clickcapture from applying on the event wich start this function to not auto-remove itself
        initialEvent.stopPropagation();
        // Reset click.clickcapture event if this function is re-called before it's done
        $(document).off("click.clickcapture");

        // Get click position
        $(document).one("click.clickcapture", function(e) {
            if (e.target.id === "click-capture-box") {
                var position = {
                    "y": (e.pageY - $(e.target).offset().top) - 14,
                    "x": (e.pageX - $(e.target).offset().left) - 8
                };

                resolve(position); // Resolve Promise (return function value)
            } else {
                reject("Wrong target clicked");
            }

            $("div#click-capture-box").remove();
        });

        $(document).on("keydown.abortclickcapture", function(e) {
            // Reset event if user press ESC on his keyboard
            if (e.keyCode == 27) {
                $(document).off("click.clickcapture");
                $("div#click-capture-box").remove();
                reject("Cancel");
            }
        });
    });
}

// Send a new object into the DB & if succeed, display it on the plan
function createNewToolObj(newObj, planId) {
    var postObj = JSON.stringify(newObj);

    $.ajax({
        type: "POST",
        url: "/plugin/plan/controllers/object_control.php?action=addObjectOnPlan&currentPlan=" + planId,
        data: {newObj: postObj},
        dataType: "html"
    })
    .done(function(data) {
        if (data.indexOf("id-") >= 0) {
            // Get the object ID from the DB
            newObj.id = data.split("-")[1];
            // Create the new object on the plan
            setToolObjectOnPlan(newObj, planId, true)
        } else if (data.indexOf("ERROR") >= 0) {
            displayToolMessage(data, true);
        }
    })
    .fail(function(jqXHR, msg) {
          displayToolMessage(msg, true);
    });
}

// Duplicate an object formated for DB & set it on the plan by click
// initialEvent => the start event wich will required a click position at next click
function duplicateObjByClick(initialEvent, object) {
    var plan = $("#plan"),
        planId = $("#currentPlan").data("id");

    plan.css("cursor", "copy");

    getClickPositionOnPlan(initialEvent, plan)
        .then(function(position) {
            // Verify object position on plan limits
            position = checkObjectOnPlanLimits(plan.width(), plan.height(), object.params.size.width, object.params.size.height, position);
            // Update duplicate object position with clicked & verifyed position
            object.params.position.y = position.y;
            object.params.position.x = position.x;

            // Create the new object
            createNewToolObj(object, planId);

            plan.css("cursor", "");
        }).catch(function() {
            plan.css("cursor", "");
        });
}

/* Text tool */
// Supprime les tags HTMl qui peuvent êtres dangereux dans les texts box
function removeHTMLTags(text) {
    var tagsWithClosure = ["iframe", "script"], // exemple: <script>monscript</script>
        tagsSingleLine = ["link", "meta", "\\?"]; // exemple: <link href="" rel="stylesheet"> (le "\\?" prends le php <?php code ?>)

    $.each(tagsWithClosure, function() {
        var regexWithClosure = new RegExp("<" + this + "[\\s\\S]*?<\/" + this + ">", "gim");

        text = text.replace(regexWithClosure, "");
    });
    $.each(tagsSingleLine, function() {
        var regexSingleLine = new RegExp("<" + this + "[\\s\\S]*?>", "gim");

        text = text.replace(regexSingleLine, "");
    });

    return text;
}

// Convertis les retours à la ligne format plain text en <br> html ou inversément
// function convertTextLinesBreak(text, convertToHTML=false, convertToPlainText=false) {
//     if (convertToHTML) {
//         text = text.replace(/(\r\n|\n\r|\r|\n)/gm, "<br>");
//     } else if (convertToPlainText) {
//         text = text.replace(/(<br[\s\S]*?>)/gm, "\n");  // Tous les <br> ou <br/> ou <br />
//     }
//
//     return text;
// }

// Converti une couleur RGB/RGBA en Hexa
function rgbToHex(color, addHashTag=false) {
    var match = color.replace(/\s/g, "").match(/^rgba?\((\d+),(\d+),(\d+)/i),
        output = color; // Si on ne match pas une couleur rgb ou rgba, on renvoie la variable passée.

    if (match && match.length === 4) {
        var rHex = ("0" + parseInt(match[1], 10).toString(16)).slice(-2),
            gHex = ("0" + parseInt(match[2], 10).toString(16)).slice(-2),
            bHex = ("0" + parseInt(match[3], 10).toString(16)).slice(-2);
        // Le résultat de match est transformée en entier puis en string base 16. On rajoute 0 devant pour être certain d'avoir un nombre sur 2 chiffres. ensuite on fait un slice -2 pour ne récupérer que les 2 chiffres de droite.

        output = rHex + gHex + bHex;
        if (addHashTag) {
            output = "#" + output;
        }
    }

    return output;
}

// Récupère l'opacité d'une couleur RGBA
function getRgbaOpacity(color) {
    var match = color.replace(/\s/g, "").match(/^rgba\(\d+,\d+,\d+,(\d?\.?\d?)/i),
        opacity = 1; // Si on ne match pas d'opacité dans la couleur, on retourne 1 car la couleur est complètement opaque.

    if (match && match.length === 2) {
        opacity = parseFloat("0" + match[1]);
        // "0" + est une sécurité pour toujours retourné un nombre meme si la valeur alpha est vide. Pour que parseFloat ne retourne pas NaN mais 0.
    }

    return opacity;
}

// Renvoie une couleur RGBA depuis une couleur hexa et/ou une opacité
function setRgba(hex, opacity=1) {
    var rgba = "rgba(0,0,0,0)", // Si on ne trouve pas de couleur, renvoie une couleur transparente
        cleanHex = hex.replace("#", ""),
        safeOpacity = parseFloat("0" + opacity), // Sécurité si opacity = "" opacity sera égale à 0
        match = [];

    if (cleanHex.length === 3) {
        // If hex shorthand value (ex: #fff)
        cleanHex = cleanHex + cleanHex;
    }
    match = cleanHex.match(/^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})/i);

    if (match && match.length === 4) {
        var r = parseInt(match[1], 16),
            g = parseInt(match[2], 16),
            b = parseInt(match[3], 16);

        rgba = "rgba(" + r + "," + g + "," + b + "," + safeOpacity + ")";
    }

    return rgba;
}

// Formate les paramètres d'un objet HTML dans le format dont ces paramètres sont stocké en DB
function getTextParams(container, textBox, textContent, textString) {
    var position = {
            "x": container.position().left,
            "y": container.position().top,
            "z": 1
        },
        size = {
            "width": textBox.outerWidth(),
            "height": textBox.outerHeight()
        },
        style = {
            "fontColor": rgbToHex(textBox.css("color")),
            "fontSize": parseInt(textBox.css("font-size")),
            "bgColor": rgbToHex(textBox.css("background-color")),
            "bgOpacity": getRgbaOpacity(textBox.css("background-color")),
            "borderColor": rgbToHex(textBox.css("border-top-color")), // jquery ne prends pas la propriété raccourcie border-color
            "borderWidth": parseInt(textBox.css("border-top-width")),
            "borderRadTopLeft": parseInt(textBox.css("border-top-left-radius")),
            "borderRadTopRight": parseInt(textBox.css("border-top-right-radius")),
            "borderRadBottomRight": parseInt(textBox.css("border-bottom-right-radius")),
            "borderRadBottomLeft": parseInt(textBox.css("border-bottom-left-radius")),
            "padTop": parseInt(textContent.css("padding-top")),
            "padRight": parseInt(textContent.css("padding-right")),
            "padBottom": parseInt(textContent.css("padding-bottom")),
            "padLeft": parseInt(textContent.css("padding-left"))
        },
        params = {
            "position": position,
            "size": size,
            "text": textString,
            "style": style
        };

    return params;
}

// Applique les paramètres size, style et text sur un text-box depuis un objet params formaté comme pour le stockage en DB
function setTextParams(params, textContent, textBox=false) {
    if (textBox) {
        textBox.css({
            "width": params.size.width || "",
            "height": params.size.height || "",
            "font-size": params.style.fontSize || "",
            "color": "#" + params.style.fontColor || "",
            "background-color": setRgba(params.style.bgColor || "", params.style.bgOpacity || ""),
            "border-color": setRgba(params.style.borderColor || "", 1),
            "border-width": params.style.borderWidth || 0,
            "border-top-left-radius": params.style.borderRadTopLeft || 0,
            "border-top-right-radius": params.style.borderRadTopRight || 0,
            "border-bottom-right-radius": params.style.borderRadBottomRight || 0,
            "border-bottom-left-radius": params.style.borderRadBottomLeft || 0
        });
    }
    textContent.css({
        "padding-top": params.style.padTop || 0,
        "padding-right": params.style.padRight || 0,
        "padding-bottom": params.style.padBottom || 0,
        "padding-left": params.style.padLeft || 0
    });
    textContent.html(params.text);
    // Avec la méthode jQuery.css, si aucune unité n'est précisée pour les tailles (em, %, ...) elle sera considérée comme en px. Si la propriété n'existe pas dans l'objet || "" permet de supprimer une propriété appliqué en js et laisse la propriété css initiale définie dans plan.css sur .plan-text-content
    // Le positionnement est appliqué au parent.
}

// Set l'événement d'ouverture de la boite d'actions d'édition
function clickOpenActionsEvent(container) {
    container.on("click.openactions", function() {
        if (container.find(".plan-text-infos").length == 0) {
            editTextObj(container);
        }
    });
}

// Evénements sur l'objet texte en mode edition
function setObjTextEventsHandler(container) {
    // Ouverture de la boite d'actions d'édition au click
    clickOpenActionsEvent(container); // On set l'événement

    container.on("dragstart", function() { // Lors du drag&drop, on ne doit pas ouvrir les actions d'édition donc on doit supprimer l'événement
        container.off("click.openactions");
    });

    container.on("dragstop", function() { // Lorsque le drag&drop s'arrète, on reset l'événement
        clickOpenActionsEvent(container);
    });
    /////////////////////////////////
}

// Au chargement de la page vue du plan, crée les objets texts de ce plan stocké dans la DB.
function setObjectTextOnPlan(plan, obj) {
	var textBox = $("<div>", {"id": obj.type.toString() + obj.id.toString() + "object", "class": "plan-text-box text-box-view"}),
		text = $("<pre>", {"class": "plan-text-content plan-text-view"});

	textBox.css({
		"top": obj.params.position.y || "",
        "left": obj.params.position.x || "",
		"width": obj.params.size.width || "",
        "height": obj.params.size.height || "",
		"font-size": obj.params.style.fontSize || "",
		"color": "#" + obj.params.style.fontColor || "",
		"background-color": setRgba(obj.params.style.bgColor || "", obj.params.style.bgOpacity || ""),
        "border-color": setRgba(obj.params.style.borderColor || "", 1),
        "border-width": obj.params.style.borderWidth || "",
        "border-top-left-radius": obj.params.style.borderRadTopLeft || "",
        "border-top-right-radius": obj.params.style.borderRadTopRight || "",
        "border-bottom-right-radius": obj.params.style.borderRadBottomRight || "",
        "border-bottom-left-radius": obj.params.style.borderRadBottomLeft || "",
    });
    text.css({
        "padding-top": obj.params.style.padTop || "",
        "padding-right": obj.params.style.padRight || "",
        "padding-bottom": obj.params.style.padBottom || "",
        "padding-left": obj.params.style.padLeft || ""
    });
	text.html(obj.params.text);
    // Avec la méthode jQuery.css, si aucune unité n'est précisée pour les tailles (em, %, ...) elle sera considérée comme en px. Si la propriété n'existe pas dans l'objet || "" permet de supprimer une propriété appliqué en js et laisse la propriété css initiale définie dans plan.css sur .plan-text-content
    // Le positionnement est appliqué au parent.

    textBox.append(text);
	plan.append(textBox);
}

// Au chargement de la page admin, crée les objets texts de ce plan stocké dans la DB.
function adm_setObjectTextOnPlan(plan, obj) {
	var textBox = $("<div>", {"class": "plan-text-box"}),
        text = $("<pre>", {"class": "plan-text-content plan-text-adm-view"}),
        container = $("<div>", {
            "id": obj.type.toString() + obj.id.toString() + "object",
            "class": "planObj objText",
            "data-id": obj.id,
            "data-type": obj.type
        });

    container.css({
        "top": obj.params.position.y || 0,
        "left": obj.params.position.x || 0
    });

    setTextParams(obj.params, text, textBox);
    dragPlanToolObj(container);
    setObjTextEventsHandler(container);

    plan.append(container);
    textBox.append(text);
    container.append(textBox);
}

// Set text object resizable
function resizeTextObj(plan, container, textBox, textEdit) {
    container.resizable({
        disabled: false,
        containment: plan,
        alsoResize: textBox, // C'est sur lui que la taille de l'objet est récupéré
        minWidth: parseInt(textBox.css("font-size")) * 1.75,
        minHeight: parseInt(textBox.css("font-size")) * 1.5,
        start: function(e, ui) {
            // Vu que le textBox doit être en box-sizing: border-box, il faut gérer les différences de tailles si il y a une bordure
            var resizeHeightCorrection = textBox.outerHeight() - textBox.height(),
                resizeWidthCorrection = textBox.outerWidth() - textBox.width();

            ui.size.height += resizeHeightCorrection;
            ui.size.width += resizeWidthCorrection;
        },
        resize: function(e, ui) {
            // Garder la hauteur du textarea a la hauteur de son contenu
            textEdit.css("height", 0); // D'abord, on set height a 0 pour reinitialiser sa valeur scrollHeigh
            textEdit.css("height", textEdit.prop("scrollHeight"));
        }
    });
}

function closeEditTextMenu(container) {
    $(".plan-text-infos", container).remove();
    $(".plan-text-edit", container).remove();
    $(".plan-text-adm-view", container).show();
    container.removeClass("edit-mode");
    container.resizable({
        disabled: true
    });
}

// Passe l'objet texte du mode visualisation en mode édition
function editTextObj(container) {
    var textEdit = $("<textarea>", {"class": "plan-text-content plan-text-edit", "placeholder": lang.PLAN_TOOLS_TEXT_PLACEHOLDER}),
        wrapper = $("<div>", {"class": "plan-text-infos"}),
        actions = $("<div>", {"class": "plan-text-actions"}),
        acceptButton = $("<button>", {"class": "text-action accept-text", "type": "button", "title": lang.PLAN_MODIFY_VALIDATE}),
        refuseButton = $("<button>", {"class": "text-action refuse-text", "type": "button", "title": lang.PLAN_MODIFY_ABORT}),
        modifyButton = $("<button>", {"class": "text-action modify-text", "type": "button", "title": lang.PLAN_MODIFY}),
        delButton = $("<button>", {"class": "text-action delete-text", "type": "button", "title": lang.PLAN_DELETE}),
        cloneButton = $("<button>", {"class": "text-action clone-text", "type": "button", "title": lang.PLAN_CLONE}),
        textBox = container.find(".plan-text-box"),
        textObj = textBox.find(".plan-text-adm-view"),
        text = textObj.html(),
        textObjParams = getTextParams(container, textBox, textObj, text);

    actions.append(modifyButton).append(cloneButton).append(delButton).append(acceptButton).append(refuseButton);
    wrapper.append(actions);
    container.append(wrapper);
    textBox.append(textEdit);

    textObj.hide();
    textEdit.focus();
    setTextParams(textObjParams, textEdit);
    // Fixer la hauteur du textarea a la hauteur de son contenu
    textEdit.css("height", textEdit.prop("scrollHeight"));
    textEdit.on("input", function() {
        $(this).css("height", $(this).prop("scrollHeight"));
    });

    container.addClass("edit-mode");

    resizeTextObj($("#plan"), container, textBox, textEdit);

    // Evénements au click sur les boutons d'action.
    $(".plan-text-actions .text-action", container).on("click", function(e) {
        // Sert a prévenir le click sur le parent (qui ouvre ce menu d'édition) pour ne pas réouvrir systématiquement la fenètre d'édition quand on click sur un bouton qui devrais la fermé.
        e.stopPropagation();
    });

    // Validation des modifications
    $(".accept-text", container).on("click", function() {
        var text = removeHTMLTags(textEdit.val()),
            params = getTextParams(container, textBox, textEdit, text);

        if ($.isNumeric(container.data("id"))) {
            var jsonParams = JSON.stringify(params),
                objId = container.data("id");

            $.ajax({
                type: "POST",
                url: "/plugin/plan/controllers/tools_control.php?action=updateObjText&object=" + objId,
                data: {params: jsonParams},
                dataType: "html"
            })
            .done(function(data) {
                if (data) { // Error Message
                    displayToolMessage(data, true);
                }
            });
        }

        setTextParams(params, textObj);
        closeEditTextMenu(container);
    });

    // Annuler les modifications
    $(".refuse-text", container).on("click", function() {
        // Rétablir les tailles initiales
        container.css({
            "width": "",
            "height": ""
        });
        textBox.css({
            "width": textObjParams.size.width,
            "height": textObjParams.size.height
        });

        closeEditTextMenu(container);
    });

    // Modifier les paramètres de l'objet texte (styles)
    $(".modify-text", container).on("click", function() {
        if ($.isNumeric($("#currentPlan").data("id")) && $.isNumeric(container.data("id"))) {
            var planId = $("#currentPlan").data("id"),
                text = removeHTMLTags(textEdit.val()),
                params = getTextParams(container, textBox, textEdit, text),
                object = JSON.stringify({
                    // encodeURIComponent est utiliser pour convertir les # des couleurs hexadecimales sinon l'objet ne passerait pas en GET. PHP reconverti automatiquement les caractères, il n'y a pas de reconvertion a faire dans le controlleur.
                    id: container.data("id"),
                    type: 5,
                    params: params
                }),
                planSize = JSON.stringify({
                    width: $("#plan").width(),
                    height: $("#plan").height()
                }),
                url = "/plugin/plan/controllers/tools_control.php?action=confObjText&plan=" + planId + "&obj=" + object + "&planSize=" + planSize,
                dialog = $("#dialog-info"),
                dialogIframe = $("<iframe>", {"src": url});

            // Création de la fenetre pop-up de configuration des paramètres de styles de l'objet texte
            dialog.dialog({
                resizable: true,
                modal: true,
                autoOpen: true,
                width: 1000,
                height: 800,
                title: lang.PLAN_TOOLS_TEXT_CONF,
                close: function() {
                    // A la fermeture de la fenetre pop-up, supprimer le dialog et son contenu pour pouvoir recréer un nouveau avec un autre contenu.
                    $(this).dialog("destroy");
                    $(this).children().remove();
                }
            });
        	dialogIframe.css({
                "width": "100%",
                "height": "100%"
            });
        	dialog.append(dialogIframe);

            closeEditTextMenu(container);
        }
    });

    // Suppression de l'objet texte
    $(".delete-text", container).on("click", function() {
        var acceptDelButton = $("<button>", {"class": "text-action accept-del-text", "type": "button", "title": lang.PLAN_DELETE_VALIDATE}),
            refuseDelButton = $("<button>", {"class": "text-action refuse-del-text", "type": "button", "title": lang.PLAN_DELETE_ABORT});

        actions.children().each(function() {
            $(this).hide();
        });
        actions.append(acceptDelButton).append(refuseDelButton);

        // Valider la suppression de l'objet texte
        $(".accept-del-text", container).on("click", function() {
            $.ajax({
                type: "GET",
                url: "/plugin/plan/controllers/object_control.php",
                data: {action: "delObj", obj: container.attr("data-id")},
                dataType: "json"
            })
            .done(function(data) {
                if (data) { //Id de l'objet supprimer. Si data est vide, c'est que l'objet n'a pas pu être supprimer.
                    container.remove();
                } else {
                    displayToolMessage(lang.PLAN_TOOLS_OBJECT_DELETE_FAILED, true);
                }
            });
        });

        // Annuler la suppression de l'objet texte
        $(".refuse-del-text", container).on("click", function() {
            acceptDelButton.remove();
            refuseDelButton.remove();
            actions.children().each(function() {
                $(this).show();
            });
        });
    });

    // Dupliquer l'objet texte
    $(".clone-text", container).on("click", function(e) {
        // Valider les éventuelles modifications sur l'object initiale avant de dupliquer
        $(".accept-text", container).trigger("click");

        var text = textObj.html(),
            cloneTextParams = getTextParams(container, textBox, textObj, text),
            objCloned = {
                type: 5,
                name: $("#currentPlan").data("id") + "-text",
                params: cloneTextParams
            };

        duplicateObjByClick(e, objCloned);
    });
}

// Créer un nouvel objet texte depuis l'outils de création de texte
function createNewObjText(position, plan) {
    count++; // Permet de différencier les objets temporaires entre eux dans le cas ou on en crée plusieurs
    var planId = $("#currentPlan").data("id"),
        textEdit = $("<textarea>", {"class": "plan-text-content plan-text-edit", "placeholder": lang.PLAN_TOOLS_TEXT_PLACEHOLDER}),
        wrapper = $("<div>", {"class": "plan-text-infos"}),
        actions = $("<div>", {"class": "plan-text-actions"}),
        acceptButton = $("<button>", {"class": "text-action accept-text", "type": "button", "title": lang.PLAN_MODIFY_VALIDATE}),
        delButton = $("<button>", {"class": "text-action delete-text", "type": "button", "title": lang.PLAN_DELETE}),
        textBox = $("<div>", {"class": "plan-text-box"}),
        tmpObj = $("<div>", {"id": "tmp-" + count, "class": "planObj objText edit-mode"}),
        name = planId + "-text";

    actions.append(acceptButton).append(delButton);
    wrapper.append(actions);
    textBox.append(textEdit);
    tmpObj.append(textBox).append(wrapper);
    tmpObj.append(wrapper);
    plan.append(tmpObj);

    position = checkObjectOnPlanLimits(plan.width(), plan.height(), textBox.width(), textBox.height(), position);

    tmpObj.css({
        "top": position.y,
        "left": position.x
    });

    dragPlanToolObj(tmpObj);
    resizeTextObj(plan, tmpObj, textBox, textEdit);

    textEdit.focus();

    // Validation des modifications
    $(".accept-text", tmpObj).on("click", function() {
        var text = removeHTMLTags(textEdit.val()),
            params = getTextParams(tmpObj, textBox, textEdit, text);
            newObj = {
                "name": name,
                "type": 5,
                "params": params
            };

        tmpObj.remove();
        createNewToolObj(newObj, planId);
    });

    // Suppression de l'objet texte
    $(".delete-text", tmpObj).on("click", function() {
        tmpObj.remove();
    });
}

// Gère les événements pour la création de texte au click sur le plan
function createTextByClick(initialEvent, textButton) {
    var plan = $("#plan");

    $(".plan-tools-bar").css("cursor", "text");
    plan.css("cursor", "text");
    textButton.addClass("active");

    getClickPositionOnPlan(initialEvent, plan)
        .then(function(position) {
            createNewObjText(position, plan);

            $(".plan-tools-bar").css("cursor", "");
            plan.css("cursor", "");
            textButton.removeClass("active");
        })
        .catch(function() {
            $(".plan-tools-bar").css("cursor", "");
            plan.css("cursor", "");
            textButton.removeClass("active");
        });
}

// Creation de la fenètre de prévisualisation sur la page pop-up de modification de l'objet texte
function createPlanTxtPrev(obj, planId, planSize) {
    var prevWindow = $("#obj-" + obj.id + "-prev"),
        preview = $(".plan-preview", prevWindow),
        plan = $("#plan" + planId, prevWindow),
        prevTextBox = $("#" + obj.type + obj.id + "object", prevWindow),
        prevTextContent = prevTextBox.find(".plan-text-content"),
        minGap = 30,
        previewXPos = 0,
        previewYPos = 0,
        previewWidth = 0,
        previewHeight = 0;

    setTextParams(obj.params, prevTextContent, prevTextBox);

    previewXPos = minGap - obj.params.position.x,
    previewYPos = minGap - obj.params.position.y,
    previewWidth = prevTextBox.outerWidth() + (minGap * 2),
    previewHeight = prevTextBox.outerHeight() + (minGap * 2);

    plan.css({
        "width": planSize.width,
        "height": planSize.height,
        "transform": "translate(" + previewXPos + "px, " + previewYPos + "px)"
    });

    preview.css({
        "min-width": previewWidth,
        "min-height": previewHeight
    });

    prevTextBox.css({
        "z-index": 300
    });

    return prevTextBox;
}

// Affiche dans la preview en temp réel les modification en cours sur l'objet texte
function planTxtPrevLiveStyles(prevTextBox) {
    var prevTextContent = prevTextBox.find(".plan-text-content");

    // Font
    $("#fontSize").on("change", function() {
        prevTextBox.css("font-size", this.value);
    });

    $("#fontColor").on("change", function() {
        prevTextBox.css("color", this.value);
    });

    // Background
    $("#bgColor, #bgOpacity").on("change", function() {
        prevTextBox.css("background-color", setRgba($("#bgColor").val(), $("#bgOpacity").val()));
    });

    // Border
    $("#borderColor").on("change", function() {
        prevTextBox.css("border-color", this.value);
    });

    $("#borderWidth").on("change", function() {
        prevTextBox.css("border-width", this.value);
    });

    // Border radius (Avec la méthode jQuery .css, pour le border radius l'unité en px est nécessaire)
    $("#topLeftRad").on("change", function() {
        prevTextBox.css("border-top-left-radius", this.value + "px");
        if ($(this).hasClass("overhide")) {
            prevTextBox.css({
                "border-top-right-radius": this.value + "px",
                "border-bottom-left-radius": this.value + "px",
                "border-bottom-right-radius": this.value + "px"
            });
        }
    });

    $("#topRightRad").on("change", function() {
        prevTextBox.css("border-top-right-radius", this.value + "px");
    });

    $("#bottLeftRad").on("change", function() {
        prevTextBox.css("border-bottom-left-radius", this.value + "px");
    });

    $("#bottRightRad").on("change", function() {
        prevTextBox.css("border-bottom-right-radius", this.value + "px");
    });

    // Padding
    $("#topPad").on("change", function() {
        prevTextContent.css("padding-top", this.value);
        if ($(this).hasClass("overhide")) {
            prevTextContent.css({
                "padding-left": this.value,
                "padding-right": this.value,
                "padding-bottom": this.value
            });
        }
    });

    $("#leftPad").on("change", function() {
        prevTextContent.css("padding-left", this.value);
    });

    $("#rightPad").on("change", function() {
        prevTextContent.css("padding-right", this.value);
    });

    $("#bottPad").on("change", function() {
        prevTextContent.css("padding-bottom", this.value);
    });
}

// Récupère les modifications tel qu'elles apparaissent dans la preview et les renvoie au controller
function sendNewTxtStyles(obj, prevTextBox) {
    var prevTextContent = prevTextBox.find(".plan-text-content"),
        params = getTextParams(prevTextBox, prevTextBox, prevTextContent, prevTextContent.html()),
        jsonParams = JSON.stringify(params);

    $.ajax({
        type: "POST",
        url: "/plugin/plan/controllers/tools_control.php?action=updateObjText&object=" + obj.id,
        data: {params: jsonParams},
        dataType: "html"
    })
    .done(function(data) {
        if (data) {
            // Message d'erreur
            displayToolMessage(data, true);
        } else {
            // On applique les nouveaux parametres sur l'objet du plan initial
            var mainWindowTextBox = window.parent.$("#" + obj.type.toString() + obj.id.toString() + "object").find(".plan-text-box"), // window.parent permet de quitter l'iframe pour revenir dans le document principale.
                mainWindowTextContent = mainWindowTextBox.find(".plan-text-content");

            setTextParams(params, mainWindowTextContent, mainWindowTextBox);
            // On ferme la fenetre pop-up
            window.parent.$("#dialog-info").dialog("close"); // C'est a partir du document principale que l'on doit appeler la méthode dialog("close").
        }
    });
}

// Déployement ou fermeture des inputs style-action au click.
function toggleStyleAction(container, forceOpen=false, forceClose=false) {
    var disabledImg = container.find(".style-action-disabled"),
        enabledImg = container.find(".style-action-enabled"),
        mainInput = container.find(".main-input"),
        othersInputs = $("input", container).not(".main-input");

    if (forceOpen) {
        container.addClass("active");
    } else if (forceClose) {
        container.removeClass("active");
    } else {
        container.toggleClass("active");
    }

    if (container.hasClass("active")) {
        disabledImg.hide();
        enabledImg.show();
        mainInput.removeClass("overhide");
        othersInputs.trigger("change");
    } else {
        enabledImg.hide();
        disabledImg.show();
        mainInput.addClass("overhide");
        mainInput.trigger("change");
    }
}

// Gère les événements liés à la modification des styles de l'objet text
function handleTxtObjUpdateStyles(obj, planId, planSize, planController) {
    // Requete ajax pour créer le plan pour la preview
    $.ajax({
        type: "GET",
        url: planController,
        data: {action: "dash_plan", id: planId},
        dataType: "html"
    })
    .done(function(data) {
        $("#obj-" + obj.id + "-prev").html(data);
        $(".plan-main").removeClass().addClass("plan-preview"); // Remove all classes from the .plan-main container & add the class for plan preview

        // Crée la fenetre de previsualisation
        prevTextBox = createPlanTxtPrev(obj, planId, planSize);

        // Au chargement du pop-up déploie ou non les inputs des style-action (Angles de la boite, espacement...). Si toutes les inputs on la même valeur, on reste en mode d'affichage a une seule input. Si une seule des valeurs est différente, on deploie les autres inputs.
        $(".style-action").each(function() {
            var container = $(this),
                mainVal = $(".main-input", container).val();

            $("input", container).each(function() {
                if ($(this).val() !== mainVal) {
                    // On désactive les transitions (pas de transition pour la position au chargement du pop-up)
                    $(".style-action-content", container).css("transition", "none");
                    $("input", container).css("transition", "none");
                    // On déploie les inputs
                    toggleStyleAction(container, true, false);
                    // On réactive les transitions (pour les changements au click)
                    $(".style-action-content", container).css("transition", "");
                    $("input", container).css("transition", "");

                    return false;
                }
            });
        });
        // Déployement ou fermeture des inputs style-action au click.
        $(".style-action-btn").on("click", function(){
            toggleStyleAction($(this).closest(".style-action"), false, false);
        });

        // Evénements de modification de styles et visualiation en directe dans la fenètre de prévisualisation
        planTxtPrevLiveStyles(prevTextBox);

        // Sauvegarder les modifications
        $("#sendFormTxtUpd").on("click", function(e) {
            e.preventDefault();

            sendNewTxtStyles(obj, prevTextBox);
        });
    });
}
/* END - Text tool */

// Handle plan tools bar events (Text, Service value, ...)
function handlePlanTools() {
    // Safety if count var hasn't been initilised. It's used to create temporary objects with differents ID's
    if (typeof count === "undefined") {
        count = 0;
    }
    // Text tool
    $(".plan-tool-text").on("click", function(e) {
        createTextByClick(e, $(this));
    });
}
